@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  *
- * $Id: mediamarks.c,v 1.42 2005/01/31 03:19:19 dsalt Exp $
+ * $Id: mediamarks.c,v 1.63 2005/11/10 18:55:29 dsalt Exp $
  *
  * mediamarks - load
  *            - save
@@ -34,6 +34,7 @@
 #include <gdk/gdk.h>
 #include <glib.h>
 
+#include "ui.h"
 #include "utils.h"
 #include "engine.h"
 #include "mediamarks.h"
@@ -42,107 +43,144 @@
 #include "play_item.h"
 
 static GtkTreeStore *mm_store;
+static GtkTreeSelection *sel;
 static GtkWidget    *manage_dlg, *cat_dlg, *cat_entry, *mm_tree;
 static GtkTreeIter   cat_iter;
 static gboolean      is_visible, cat_edited;
 
-static void menu_cb (GtkWidget* widget, gpointer data) {
+static GtkUIManager *mm_ui;
+static GtkActionGroup *media_group = NULL;
+static guint	       media_merge;
 
-  play_item_t *item_orig = (play_item_t *) data;
+static void mm_add_at_selected (play_item_t *play_item);
 
-  if (item_orig) {
-    play_item_t *item;
-    int          pos;
-
-    item = play_item_copy (item_orig);
-
-    pos = playlist_add (item, -1);
-    playlist_play (pos);
-  }
+static void menu_cb (GtkAction* action, gpointer data)
+{
+  if (data)
+    playlist_play (playlist_add (play_item_copy (data), -1));
 }
 
-static void gen_items (GtkMenuShell *submenu, GtkTreeIter *iter) {
-
-  int pos;
-
-  pos = 4;
-
+static int gen_actions_sub (GtkUIManager *ui, GtkActionGroup *actions,
+			    GtkTreeIter *iter, int count)
+{
   do {
+    GValue v = {0};
+    char  *id;
 
-    GValue       v;
-    play_item_t *play_item;
-    GtkWidget    *item;
-    char         *id;
-      
-    memset (&v, 0, sizeof (GValue));
-    gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store),
-			      iter, 0, &v);
+    gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store), iter, 0, &v);
     id = g_value_peek_pointer (&v);
 
     if (id) {
-      item = gtk_menu_item_new_with_label (id);
+      char *name = g_strdup_printf ("_mediamark_%d", count++);
+      GtkAction *action = gtk_action_new (name, id, NULL, NULL);
+      play_item_t *item;
+
+      gtk_action_group_add_action (actions, action);
+      free (name);
+
       g_value_unset (&v);
-
-      memset (&v, 0, sizeof (GValue));
-      gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store),
-				iter, 2, &v);
-      play_item = g_value_peek_pointer (&v);
+      gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store), iter, 2, &v);
+      item = g_value_peek_pointer (&v);
       
-      if (play_item) {
-	
-	gtk_signal_connect (GTK_OBJECT (item), "activate", 
-			    GTK_SIGNAL_FUNC (menu_cb), (gpointer) play_item);
-	
-      } else {
-	
-	GtkWidget   *submenu;
-	GtkTreeIter  children;
-	
-	submenu = gtk_menu_new ();
-	
-	if (gtk_tree_model_iter_children (GTK_TREE_MODEL(mm_store), &children, iter))
-	  gen_items (GTK_MENU_SHELL(submenu), &children);
-	
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), submenu);
-	
+      if (item)
+	g_signal_connect (G_OBJECT(action), "activate",
+			  G_CALLBACK(menu_cb), (gpointer) item);
+      else
+      {
+	GtkTreeIter children;
+	if (gtk_tree_model_iter_children (GTK_TREE_MODEL(mm_store),
+					  &children, iter))
+	  count = gen_actions_sub (ui, actions, &children, count);
       }
-      
-      gtk_menu_shell_insert (submenu, item, pos);
-      gtk_widget_show(item);
-
-      pos++;
     }
 
     g_value_unset (&v);
-      
   } while (gtk_tree_model_iter_next (GTK_TREE_MODEL(mm_store), iter));
+
+  return count;
 }
 
-static void update_menu (GtkMenuShell *submenu) {
+static GtkActionGroup *gen_actions (GtkUIManager *ui, GtkTreeIter *iter)
+{
+  GtkActionGroup *actions = gtk_action_group_new ("mediamarks");
+#ifdef ENABLE_NLS
+  gtk_action_group_set_translation_domain (actions, "");
+#endif
+  gen_actions_sub (ui, actions, iter, 0);
+  return actions;
+}
 
-  GList        *olditems;
-  GtkWidget    *item;
+static int gen_items (GtkUIManager *ui, const char *path, GtkTreeIter *iter,
+		      guint merge, int count)
+{
+  do {
+    GValue v = {0};
+    char  *id;
+
+    gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store), iter, 0, &v);
+    id = g_value_peek_pointer (&v);
+
+    if (id) {
+      char *name = g_strdup_printf ("_mediamark_%d", count++);
+      play_item_t *item;
+
+      g_value_unset (&v);
+      gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store), iter, 2, &v);
+      item = g_value_peek_pointer (&v);
+      
+      if (item)
+	gtk_ui_manager_add_ui (ui, merge, path, name, name,
+			       GTK_UI_MANAGER_MENUITEM, FALSE); 
+      else
+      {
+	GtkTreeIter children;
+	gtk_ui_manager_add_ui (ui, merge, path, name, name,
+			       GTK_UI_MANAGER_MENU, FALSE); 
+	if (gtk_tree_model_iter_children (GTK_TREE_MODEL(mm_store),
+					  &children, iter))
+	{
+	  char *newpath = g_strdup_printf ("%s/%s", path, name);
+	  count = gen_items (ui, newpath, &children, merge, count);
+	  free (newpath);
+	}
+      }
+    }
+
+    g_value_unset (&v);
+  } while (gtk_tree_model_iter_next (GTK_TREE_MODEL(mm_store), iter));
+
+  return count;
+}
+
+static void update_menu (void)
+{
   GtkTreeIter   iter;
 
-  if (!submenu)
-    return;
-
-  /*
-   * remove old items (if any)
-   */
-
-  olditems = gtk_container_children (GTK_CONTAINER(submenu));
-  while ( (item = (GtkWidget *) g_list_nth_data (olditems, 4)) ) {
-    gtk_container_remove (GTK_CONTAINER (submenu), item);
-    olditems = gtk_container_children (GTK_CONTAINER (submenu));
+  /* remove old items (if any) */
+  if (media_group)
+  {
+    gtk_ui_manager_remove_ui (ui, media_merge);
+    gtk_ui_manager_remove_action_group (ui, media_group);
+    media_group = NULL;
   }
 
-  /*
-   * add current items
-   */
+  /* add current items */
 
   if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (mm_store), &iter))
-    gen_items (submenu, &iter);
+  {
+    int i;
+    const char *tree;
+    media_merge = gtk_ui_manager_new_merge_id (ui);
+    media_group = gen_actions (ui, &iter);
+    gtk_ui_manager_insert_action_group (ui, media_group, 0);
+    for (i = 0; (tree = menu_get_tree_name (i)) != NULL; ++i)
+    {
+      char *key = g_strdup_printf ("/ui/%s/MediaMenu/MediaItems", tree);
+      gtk_tree_model_get_iter_first (GTK_TREE_MODEL (mm_store), &iter);
+      gen_items (ui, key, &iter, media_merge, 0);
+      free (key);
+    }
+  }
 }
 
 /* 
@@ -152,8 +190,7 @@ static void update_menu (GtkMenuShell *submenu) {
 static void get_items (xml_node_t *node, int depth, GtkTreeIter *parent) {
   /* assumption: *one* of (depth == 0) or (parent == NULL) */
 
-  while (node) {
-
+  foreach_glist (node, node)
     if (!strcasecmp (node->name, "sub")) {
 
       GtkTreeIter iter;
@@ -175,11 +212,8 @@ static void get_items (xml_node_t *node, int depth, GtkTreeIter *parent) {
 			  play_item->mrl, 2, play_item, -1);
 
     } else {
-      printf (_("mediamarks: error, unknown node type %s\n"), node->name);
+      fprintf (stderr, _("mediamarks: error: unknown node type %s\n"), node->name);
     }
-
-    node = node->next;
-  }
 }
 
 static void load_new_file (xml_node_t *node) {
@@ -188,7 +222,7 @@ static void load_new_file (xml_node_t *node) {
 
     get_items (node->child, 0, NULL);
   } else {
-    printf (_("mediamarks: ERROR, root node must be GXINEMM\n"));
+    fprintf (stderr, _("mediamarks: error: root node must be GXINEMM\n"));
   }
 }
 
@@ -216,79 +250,85 @@ static char * read_line (FILE *f) {
 static void load_old_file (char *fname) {
 
   FILE        *f;
-  int          depth;
   GtkTreeIter  parent;
   GtkTreeIter  iter;
 
   gtk_tree_model_get_iter_first ( GTK_TREE_MODEL (mm_store), &parent);
 
   f = fopen (fname, "r");
-  if (f) {
-    
-    depth = 0;
+  if (f)
+  {
+    int depth = 0;
 
-    while (1) {
-      char *id, *str;
-
-      id = read_line (f);
+    for (;;)
+    {
+      char *id = read_line (f);
       if (!id)
 	break;
 
-      if (!strncmp (id, "<sub>", 5)) {
-
+      if (!strncmp (id, "<sub>", 5))
+      {
 	gtk_tree_store_append (mm_store, &iter, depth ? &parent : NULL);
 	gtk_tree_store_set (mm_store, &iter, 0, &id[5], -1);
 
 	parent = iter;
 	depth++;
-
-      } else if (!strncmp (id, "</sub>", 6)) {
-      
+      }
+      else if (!strncmp (id, "</sub>", 6))
+      {
 	/* sub-tree ends here */
-
 	gtk_tree_model_iter_parent (GTK_TREE_MODEL (mm_store), 
 				    &iter, &parent);
-	depth--;
-	if (depth<0) {
-
-	  printf (_("mediamarks: media marks file %s corrupted - ignoring rest of file\n"),
-		  fname);
+	if (--depth < 0)
+	{
+	  fprintf (stderr, _("mediamarks: media marks file %s corrupted - ignoring rest of file\n"),
+		   fname);
+	  free (id);
 	  return;
 	}
 	parent = iter;
-
-      } else { 
-	
+      }
+      else
+      { 
 	play_item_t *play_item;
-	char        *mrl;
+	char        *mrl, *str;
 	int          time = 0;
 
 	mrl = read_line (f);
 	if (!mrl)
+	{
+	  free (id);
 	  break;
+        }
 
 	str = read_line (f);
-	if (!str)
+	{
+	  free (mrl);
+	  free (id);
 	  break;
-	sscanf (str, "%d", &time);
+        }
 
+	sscanf (str, "%d", &time);
 	play_item = play_item_new (id, mrl, time);
 
 	gtk_tree_store_append (mm_store, &iter, depth ? &parent : NULL);
 	gtk_tree_store_set (mm_store, &iter, 0, id, 1, mrl, 2, play_item, -1);
+
+	free (str);
+	free (mrl);
       }
 
       /* discard newline */
-      str = read_line (f);
-      if (!str)
+      free (id);
+      id = read_line (f);
+      if (!id)
 	break;
+      free (id);
     }
     fclose (f);
-  } else {
-
-    printf (_("mediamarks: load failed!\n"));
-
   }
+  else
+    fprintf (stderr, _("mediamarks: load failed!\n"));
 }
 
 static void cat_response_cb (GtkDialog* widget, int response, gpointer data)
@@ -306,35 +346,30 @@ static void cat_response_cb (GtkDialog* widget, int response, gpointer data)
 static gboolean do_category_edit (void)
 {
   cat_edited = FALSE;
-  gtk_widget_show_all (cat_dlg);
-  if (!GTK_WINDOW(cat_dlg)->modal)
-    gtk_window_set_modal (GTK_WINDOW(cat_dlg), TRUE);
+  gtk_window_set_modal (GTK_WINDOW(cat_dlg), TRUE);
   gtk_widget_grab_focus (cat_entry);
+  window_show (cat_dlg, manage_dlg);
 
   gtk_main();
 
   return cat_edited;
 }
 
-static void edit_cb (GtkWidget* widget, gpointer data) {
+static void edit_cb (GtkWidget* widget, gpointer data)
+{
+  GtkTreeIter iter;  
 
-  GtkTreeView       *tree = GTK_TREE_VIEW (mm_tree);
-  GtkTreeSelection  *sel;
-  GtkTreeIter        iter;  
-
-  sel = gtk_tree_view_get_selection (tree);
-
-  if (gtk_tree_selection_get_selected (sel, NULL, &iter)) {
-    GValue       v;
+  if (gtk_tree_selection_get_selected (sel, NULL, &iter))
+  {
+    GValue       v = {0};
     play_item_t *play_item;
 
-    memset (&v, 0, sizeof (GValue));
     gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store),
 			      &iter, 2, &v);
     play_item = g_value_peek_pointer (&v);
     if (play_item) {
       logprintf ("mediamarks: got a play item\n");
-      if (play_item_edit (play_item, PLAY_ITEM_MEDIAMARK, NULL))
+      if (play_item_edit (play_item, PLAY_ITEM_MEDIAMARK, NULL, manage_dlg))
       {
 	logprintf ("mediamarks: play item title after edit : %s\n",
 		   play_item->title);
@@ -343,12 +378,11 @@ static void edit_cb (GtkWidget* widget, gpointer data) {
 			    1, play_item->mrl, 2, play_item, -1);
       }
     } else {
-      GValue       v;
+      GValue       v = {0};
       char        *name;
 
       memcpy (&cat_iter, &iter, sizeof (cat_iter));
 
-      memset (&v, 0, sizeof (GValue));
       gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store),
 				&iter, 0, &v);
       name = (char *) g_value_peek_pointer (&v);
@@ -363,22 +397,33 @@ static void edit_cb (GtkWidget* widget, gpointer data) {
   }
 }
 
-static void delete_cb (GtkWidget* widget, gpointer data) {
-
-  GtkTreeView       *tree = GTK_TREE_VIEW (mm_tree);
-  GtkTreeSelection  *sel;
-  GtkTreeIter        iter;  
-
-  sel = gtk_tree_view_get_selection (tree);
-
+static void delete_cb (GtkWidget* widget, gpointer data)
+{
+  GtkTreeIter iter;  
   if (gtk_tree_selection_get_selected (sel, NULL, &iter))
     gtk_tree_store_remove (mm_store, &iter);
 }
 
-static void new_category_cb (GtkWidget* widget, gpointer data) {
+static void copy_cb (GtkWidget* widget, gpointer data)
+{
+  clip_set_play_item_from_selection (mm_tree);
+}
 
-  GtkTreeView       *tree = GTK_TREE_VIEW (mm_tree);
-  GtkTreeSelection  *sel;
+static void cut_cb (GtkWidget* widget, gpointer data)
+{
+  copy_cb (widget, data);
+  delete_cb (widget, data);
+}
+
+static void paste_cb (GtkWidget* widget, gpointer data)
+{
+  play_item_t *item = clip_get_play_item ();
+  if (item)
+    mm_add_at_selected (item);
+}
+
+static void new_category_cb (GtkWidget* widget, gpointer data)
+{
   GtkTreeIter        iter, parent;   
   gboolean           in_root = TRUE;
 
@@ -386,17 +431,14 @@ static void new_category_cb (GtkWidget* widget, gpointer data) {
    * find out where to create the new section
    */
 
-  sel     = gtk_tree_view_get_selection (tree);
-
   if (gtk_tree_selection_get_selected (sel, NULL, &iter)) {
-    GValue       v;
+    GValue       v = {0};
     play_item_t *play_item;
 
     /*
      * is this a section or a play item ?
      */
 
-    memset (&v, 0, sizeof (GValue));
     gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store),
 			      &iter, 2, &v);
     play_item = g_value_peek_pointer (&v);
@@ -407,7 +449,7 @@ static void new_category_cb (GtkWidget* widget, gpointer data) {
       GtkTreePath *path;
       path = gtk_tree_model_get_path (GTK_TREE_MODEL (mm_store), &iter);
 
-      if (gtk_tree_view_row_expanded (tree, path)) {
+      if (gtk_tree_view_row_expanded (GTK_TREE_VIEW (mm_tree), path)) {
 	in_root = 0;
 	memcpy (&parent, &iter, sizeof (iter));
       } else {
@@ -435,11 +477,10 @@ static void new_category_cb (GtkWidget* widget, gpointer data) {
   }
 }
 
-static void tree_changed_cb (GtkTreeModel *treemodel,
-			     GtkTreePath *arg1,
-			     gpointer user_data) {
-  update_menu (media_menu[0]);
-  update_menu (media_menu[1]);
+static void tree_changed_cb (GtkTreeModel *treemodel, GtkTreePath *arg1,
+			     gpointer user_data)
+{
+  update_menu ();
 }
 
 static int load_mediamarks (gchar *fname)
@@ -457,8 +498,7 @@ static int load_mediamarks (gchar *fname)
     } else 
       load_new_file (node);
 
-    update_menu (media_menu[0]);
-    update_menu (media_menu[1]);
+    update_menu ();
     g_signal_handlers_unblock_by_func (G_OBJECT(mm_store), tree_changed_cb, NULL);
 
     free (mmfile);
@@ -468,32 +508,28 @@ static int load_mediamarks (gchar *fname)
   return 1;
 }
 
-void mm_import (void) {
-
-  gchar *fname;
-
-  fname = modal_file_dialog (_("Select media marks file to load..."), NULL);
-
-  if (fname) {
-
+void mm_import (void)
+{
+  gchar *fname = modal_file_dialog (_("Select media marks file to load..."),
+				    TRUE, TRUE, FALSE, NULL, NULL, manage_dlg);
+  if (fname)
+  {
     if (!load_mediamarks (fname) && errno != ENOENT)
-      display_error (_("Media marks load failed"),
+      display_error (FROM_GXINE, _("Media marks load failed."),
 		     _("Couldn't load media marks file %s\n%s"),
 		     fname, strerror (errno));
+    free (fname);
   }
 }
 
-void mm_add_show (void)
+void mm_add_show (void *data)
 {
   play_item_t *play_item;
-  GtkTreeIter  iter;
   int          pos, pos_time, len;
   char	      *dot;
   const char  *title = xine_get_meta_info (stream, XINE_META_INFO_TITLE);
 
-  play_item = playlist_get_item (playlist_get_list_pos());
-  play_item = play_item ? play_item_copy (play_item)
-			: play_item_new ("", "", 0);
+  play_item = playlist_get_current_item () ? : play_item_new ("", "", 0);
 
   /* default title? try to get title from meta-info */
   dot = strrchr (play_item->mrl, '.');
@@ -510,24 +546,50 @@ void mm_add_show (void)
   xine_get_pos_length (stream, &pos, &pos_time, &len);
   play_item->start_time = (len >= 0) ? pos_time : 0;
 
-  if (play_item_edit (play_item, PLAY_ITEM_MEDIAMARK_NEW, title))
-  {
-    gtk_tree_store_append (mm_store, &iter, NULL);
-    gtk_tree_store_set (mm_store, &iter, 0, play_item->title,
-			1, play_item->mrl, 2, play_item, -1);
-  }
+  if (play_item_edit (play_item, PLAY_ITEM_MEDIAMARK_NEW, title,
+		      data ? manage_dlg : NULL))
+    mm_add (play_item);
+  else
+    play_item_dispose (play_item);
 }
 
+void mm_add (play_item_t *play_item)
+{
+  GtkTreeIter iter;
+  gtk_tree_store_append (mm_store, &iter, NULL);
+  gtk_tree_store_set (mm_store, &iter, 0, play_item->title,
+		      1, play_item->mrl, 2, play_item, -1);
+}
 
-void mm_manage_show () {
+static void mm_add_at_selected (play_item_t *play_item)
+{
+  GtkTreeIter iter, selected;
+  if (gtk_tree_selection_get_selected (sel, NULL, &selected))
+  {
+    GValue v = {0};
+    gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store), &selected, 2, &v);
+    if (g_value_peek_pointer (&v) == NULL)
+      gtk_tree_store_insert_after (mm_store, &iter, &selected, NULL);
+    else
+      gtk_tree_store_insert_after (mm_store, &iter, NULL, &selected);
 
+    gtk_tree_store_set (mm_store, &iter, 0, play_item->title,
+		        1, play_item->mrl, 2, play_item, -1);
+    g_value_unset (&v);
+  }
+  else
+    mm_add (play_item);
+}
+
+void mm_manage_show (void)
+{
   if (is_visible) {
     is_visible = FALSE;
     gtk_widget_hide (manage_dlg);
     mm_save();
   } else {
     is_visible = TRUE;
-    gtk_widget_show_all (manage_dlg);
+    window_show (manage_dlg, NULL);
   }
 }
 
@@ -540,34 +602,34 @@ static void print_depth (FILE *f, int depth) {
   }
 }
 
-static void save_items (FILE *f, int depth, GtkTreeIter *iter) {
-
-  do {
-    
-    GValue        v;
+static void save_items (FILE *f, int depth, GtkTreeIter *iter)
+{
+  do
+  {
+    GValue        v = {0};
     play_item_t  *play_item;
     char         *id;
       
-    memset (&v, 0, sizeof (GValue));
     gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store),
 			      iter, 0, &v);
     id = g_value_peek_pointer (&v);
 
-    if (id) {
+    if (id)
+    {
+      GValue vi = {0};
 
-      GValue vi;
-
-      memset (&vi, 0, sizeof (GValue));
       gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store),
 				iter, 2, &vi);
       play_item = g_value_peek_pointer (&vi);
       
-      if (play_item) {
-	
-	play_item_save (play_item, f, depth);
-	
-      } else {
-
+      if (play_item)
+      {
+        char *xml = play_item_xml (play_item, depth);
+        fputs (xml, f);
+        free (xml);
+      }
+      else
+      {
 	GtkTreeIter children;
 
 	print_depth (f, depth);
@@ -578,7 +640,6 @@ static void save_items (FILE *f, int depth, GtkTreeIter *iter) {
 	
 	print_depth (f, depth);
 	fprintf (f, "</SUB>\n");
-	
       }
       
       g_value_unset (&vi);
@@ -610,51 +671,45 @@ void mm_save (void) {
     fprintf (f, "</GXINEMM>\n");
 
     if (ferror (f))
-      printf (_("mediamarks: save to '%s' failed: %s\n"),
-	      fname, strerror (errno));
+      display_error_modal (FROM_GXINE, _("Failed to save media marks"),
+			   _("Error while writing to '%s': %s"),
+			   fname, strerror (errno));
     if (fclose (f))
-      printf (_("mediamarks: save to '%s' failed: %s\n"),
-	      fname, strerror (errno));
-  } else
-    printf (_("mediamarks: save to '%s' failed: %s\n"),
-	    fname, strerror (errno));
+      display_error_modal (FROM_GXINE, _("Failed to save media marks"),
+			   _("Error when closing '%s': %s"),
+			   fname, strerror (errno));
+  } else 
+    display_error_modal (FROM_GXINE, _("Failed to save media marks"),
+                         _("Can't open file '%s': %s"),
+			 fname, strerror (errno));
 
   g_free(fname);
 }
 
 static JSBool js_mm_add_show (JSContext *cx, JSObject *obj, uintN argc, 
-			      jsval *argv, jsval *rval) {
-
+			      jsval *argv, jsval *rval)
+{
   /* se_t *se = (se_t *) JS_GetContextPrivate(cx); */
-
-  se_log_fncall ("mm_add_show");
-
-  mm_add_show ();
-  
+  se_log_fncall_checkinit ("mm_add_show");
+  mm_add_show (NULL);
   return JS_TRUE;
 }
 
 static JSBool js_mm_manage_show (JSContext *cx, JSObject *obj, uintN argc, 
-				 jsval *argv, jsval *rval) {
-
+				 jsval *argv, jsval *rval)
+{
   /* se_t *se = (se_t *) JS_GetContextPrivate(cx); */
-
-  se_log_fncall ("mm_manage_show");
-
+  se_log_fncall_checkinit ("mm_manage_show");
   mm_manage_show ();
-  
   return JS_TRUE;
 }
 
 static JSBool js_import_mediamarks (JSContext *cx, JSObject *obj, uintN argc, 
-				    jsval *argv, jsval *rval) {
-
+				    jsval *argv, jsval *rval)
+{
   /* se_t *se = (se_t *) JS_GetContextPrivate(cx); */
-
-  se_log_fncall ("import_mediamarks");
-
+  se_log_fncall_checkinit ("import_mediamarks");
   mm_import ();
-  
   return JS_TRUE;
 }
 
@@ -671,18 +726,141 @@ static void mm_response_cb (GtkDialog *dbox, int response, gpointer data)
   }
 }
 
-void mediamarks_init (void) {
+static const GtkActionEntry buttons_data[] = {
+  { "new",	GTK_STOCK_NEW,		NULL,	NULL,		N_("Add an MRL"),		mm_add_show },
+  { "newcat",	GTK_STOCK_DIRECTORY,	NULL,	"<Control>F",	N_("Add a category folder"),	new_category_cb },
+  { "edit",	GTK_STOCK_EDIT,		NULL,	"<Control>E",	N_("Edit the selected item"),	edit_cb },
+  { "delete",	GTK_STOCK_DELETE,	NULL,	"Delete",	N_("Delete the selected item"),	delete_cb },
+  { "copy",	GTK_STOCK_COPY,		NULL,	NULL,		NULL,				copy_cb },
+  { "cut",	GTK_STOCK_CUT,		NULL,	NULL,		NULL,				cut_cb },
+  { "paste",	GTK_STOCK_PASTE,	NULL,	NULL,		NULL,				paste_cb },
+};
 
-  GtkWidget            *tree, *b, *scrolled_window, *hbox, *label;
+static const char buttons_structure[] =
+  "<ui>\n"
+    "<toolbar>\n"
+      "<toolitem action='new' />\n"
+      "<toolitem action='newcat' />\n"
+      "<toolitem action='edit' />\n"
+      "<toolitem action='delete' />\n"
+      "<toolitem action='copy' />\n"
+      "<toolitem action='cut' />\n"
+      "<toolitem action='paste' />\n"
+    "</toolbar>\n"
+  "</ui>";
+
+static void sel_changed_cb (GtkTreeSelection *sel, gpointer data)
+{
+  static const char *const items[] = { "edit", "delete", NULL };
+  static const char *const cb_items[] = { "copy", "cut", NULL };
+  GtkTreeIter iter;
+  gboolean have_selected = gtk_tree_selection_get_selected (sel, NULL, &iter);
+
+  ui_mark_active (mm_ui, items, have_selected);
+  if (have_selected)
+  {
+    GValue v = {0};
+    gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store), &iter, 2, &v);
+    ui_mark_active (mm_ui, cb_items, g_value_peek_pointer (&v) != NULL);
+    g_value_unset (&v);
+  }
+  else
+    ui_mark_active (mm_ui, cb_items, FALSE);
+}
+
+void mediamarks_init (void)
+{
+  GtkWidget            *tree, *scrolled_window, *hbox, *label;
+  GtkBox	       *vbox;
   gchar                *fname;
   GtkCellRenderer      *cell;
   GtkTreeViewColumn    *column;
+  GError	       *error;
    
   /*
    * init tree store
    */
  
   mm_store = gtk_tree_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);
+  g_object_connect (G_OBJECT (mm_store),
+	"signal::row-changed", G_CALLBACK (tree_changed_cb), NULL,
+	"signal::row-deleted", G_CALLBACK (tree_changed_cb), NULL,
+	"signal::row-has-child-toggled", G_CALLBACK (tree_changed_cb), NULL,
+	"signal::row-inserted", G_CALLBACK (tree_changed_cb), NULL,
+	"signal::rows-reordered", G_CALLBACK (tree_changed_cb), NULL,
+	NULL);
+
+  /*
+   * create (for now invisible) mm_manage dialog
+   */
+
+  manage_dlg = gtk_dialog_new_with_buttons (_("Manage media marks..."), NULL, 0,
+				GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+				GTK_STOCK_CLOSE, GTK_RESPONSE_DELETE_EVENT,
+				NULL);
+  gtk_window_set_default_size (GTK_WINDOW (manage_dlg), 400, 250);
+  g_signal_connect (G_OBJECT(manage_dlg), "response",
+		    G_CALLBACK(mm_response_cb), NULL);
+  hide_on_delete (manage_dlg, &is_visible);
+
+  vbox = GTK_BOX (GTK_DIALOG (manage_dlg)->vbox);
+
+  /* add a nice toolbar and tree view widget here */
+
+  mm_tree = tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL(mm_store));
+  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tree), TRUE);
+
+  sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
+  g_signal_connect (G_OBJECT (sel), "changed",
+		    G_CALLBACK (sel_changed_cb), NULL);
+
+  /* buttons */
+
+  mm_ui = ui_create_manager ("mm", manage_dlg);
+  gtk_action_group_add_actions (ui_get_action_group (mm_ui), buttons_data,
+				G_N_ELEMENTS (buttons_data), tree);
+  gtk_ui_manager_add_ui_from_string (mm_ui, buttons_structure, -1, &error);
+  gtk_box_pack_start (vbox, gtk_ui_manager_get_widget (mm_ui, "/toolbar"),
+		      FALSE, FALSE, 0);
+  gtk_action_group_connect_accelerators (ui_get_action_group (mm_ui));
+
+  sel_changed_cb (sel, NULL);
+
+  /* fill in and insert the tree view widget */
+
+  cell = gtk_cell_renderer_text_new ();
+  g_object_set (G_OBJECT(cell),
+		"ellipsize", 2, "width_chars", 16, NULL); /* foo...bar */
+  column = gtk_tree_view_column_new_with_attributes (_("Media mark"), cell,
+						     "text", 0, NULL);
+  gtk_tree_view_column_set_resizable (GTK_TREE_VIEW_COLUMN(column), TRUE);
+  gtk_tree_view_append_column (GTK_TREE_VIEW (tree),
+			       GTK_TREE_VIEW_COLUMN (column));
+
+  cell = gtk_cell_renderer_text_new ();
+  g_object_set (G_OBJECT(cell),
+		"ellipsize", 1, "width_chars", 16, NULL); /* ...bar */
+  column = gtk_tree_view_column_new_with_attributes (_("MRL"), cell,
+						     "text", 1, NULL);
+  gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN(column),
+				   GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+  gtk_tree_view_append_column (GTK_TREE_VIEW (tree),
+			       GTK_TREE_VIEW_COLUMN (column));
+
+  gtk_tree_view_set_reorderable (GTK_TREE_VIEW (tree), TRUE);
+
+  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
+				  GTK_POLICY_AUTOMATIC, 
+				  GTK_POLICY_AUTOMATIC);
+  gtk_container_add (GTK_CONTAINER (scrolled_window), tree);
+
+  g_signal_connect (G_OBJECT (tree), "row-activated",
+		    G_CALLBACK (edit_cb), NULL);
+
+  gtk_box_pack_start (vbox, scrolled_window, TRUE, TRUE, 2);
+
+  is_visible = FALSE;
 
   /* 
    * now parse the mediamarks file
@@ -704,103 +882,7 @@ void mediamarks_init (void) {
 
   g_free(fname);
 
-  g_signal_connect (G_OBJECT (mm_store), "row-changed",
-		    G_CALLBACK (tree_changed_cb), NULL);
-  g_signal_connect (G_OBJECT (mm_store), "row-deleted",
-		    G_CALLBACK (tree_changed_cb), NULL);
-  g_signal_connect (G_OBJECT (mm_store), "row-has-child-toggled",
-		    G_CALLBACK (tree_changed_cb), NULL);
-  g_signal_connect (G_OBJECT (mm_store), "row-inserted",
-		    G_CALLBACK (tree_changed_cb), NULL);
-  g_signal_connect (G_OBJECT (mm_store), "rows-reordered",
-		    G_CALLBACK (tree_changed_cb), NULL);
-
-  update_menu (media_menu[0]);
-  update_menu (media_menu[1]);
-
-  /*
-   * create (for now invisible) mm_manage dialog
-   */
-
-  manage_dlg = gtk_dialog_new_with_buttons (_("Manage media marks..."), NULL, 0,
-				GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
-				GTK_STOCK_CLOSE, GTK_RESPONSE_DELETE_EVENT,
-				NULL);
-  gtk_window_set_default_size (GTK_WINDOW (manage_dlg), 400, 250);
-  g_signal_connect (GTK_OBJECT(manage_dlg), "response",
-		    G_CALLBACK(mm_response_cb), NULL);
-  hide_on_delete (manage_dlg, &is_visible);
-
-  /* add a nice tree view widget here */
-
-  mm_tree = tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL(mm_store));
-  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tree), TRUE);
-
-  cell = gtk_cell_renderer_text_new ();
-  if (GTK_TEST_COMPAT_2_6)
-    g_object_set (G_OBJECT(cell),
-		  "ellipsize", 2, "width_chars", 16, NULL); /* foo...bar */
-  column = gtk_tree_view_column_new_with_attributes (_("Media mark"), cell,
-						     "text", 0, NULL);
-  gtk_tree_view_column_set_resizable (GTK_TREE_VIEW_COLUMN(column), TRUE);
-  gtk_tree_view_append_column (GTK_TREE_VIEW (tree),
-			       GTK_TREE_VIEW_COLUMN (column));
-
-  if (GTK_TEST_COMPAT_2_6)
-  {
-    cell = gtk_cell_renderer_text_new ();
-    g_object_set (G_OBJECT(cell),
-		  "ellipsize", 1, "width_chars", 16, NULL); /* ...bar */
-  }
-  column = gtk_tree_view_column_new_with_attributes ("MRL", cell,
-						     "text", 1, NULL);
-  gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN(column),
-				   GTK_TREE_VIEW_COLUMN_AUTOSIZE);
-  gtk_tree_view_append_column (GTK_TREE_VIEW (tree),
-			       GTK_TREE_VIEW_COLUMN (column));
-
-  gtk_tree_view_set_reorderable (GTK_TREE_VIEW (tree), TRUE);
-
-  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
-				  GTK_POLICY_AUTOMATIC, 
-				  GTK_POLICY_AUTOMATIC);
-  gtk_container_add (GTK_CONTAINER (scrolled_window), tree);
-
-  g_signal_connect (GTK_OBJECT (tree), "row-activated",
-		    G_CALLBACK (edit_cb), NULL);
-
-  /* add edit button bar */
-  hbox = gtk_hbox_new (FALSE, 2);
-
-  b = gtk_button_new_with_mnemonic (_("_New media mark"));
-  gtk_box_pack_start (GTK_BOX(hbox), b, FALSE, FALSE, 0);
-  g_signal_connect (GTK_OBJECT(b), "clicked",
-		    G_CALLBACK(mm_add_show), NULL);
-
-  b = gtk_button_new_with_mnemonic (_("New _category"));
-  gtk_box_pack_start (GTK_BOX(hbox), b, FALSE, FALSE, 0);
-  g_signal_connect (GTK_OBJECT(b), "clicked",
-		    G_CALLBACK(new_category_cb), tree);
-
-  b = GTK_COMPAT_2_6 (gtk_button_new_from_stock ("gtk-edit"),
-		      gtk_button_new_with_mnemonic (_("_Edit")));
-  gtk_box_pack_start (GTK_BOX(hbox), b, FALSE, FALSE, 0);
-  g_signal_connect (GTK_OBJECT(b), "clicked",
-		    G_CALLBACK(edit_cb), NULL);
-
-  b = gtk_button_new_from_stock (GTK_STOCK_DELETE);
-  gtk_box_pack_start (GTK_BOX(hbox), b, FALSE, FALSE, 0);
-  g_signal_connect (GTK_OBJECT(b), "clicked",
-		    G_CALLBACK(delete_cb), NULL);
-
-  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(manage_dlg)->vbox), hbox,
-		    FALSE, FALSE, 2);
-
-  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(manage_dlg)->vbox), scrolled_window,
-		      TRUE, TRUE, 2);
-
-  is_visible = FALSE;
+  update_menu ();
 
   /*
    * create (for now invisible) category name entry dialog
@@ -811,7 +893,7 @@ void mediamarks_init (void) {
 				GTK_STOCK_OK, GTK_RESPONSE_OK,
 				NULL);
   hide_on_delete (cat_dlg, NULL);
-  g_signal_connect (GTK_OBJECT (cat_dlg), "response",
+  g_signal_connect (G_OBJECT (cat_dlg), "response",
 		    G_CALLBACK (cat_response_cb), NULL);
   gtk_dialog_set_default_response (GTK_DIALOG(cat_dlg), GTK_RESPONSE_OK);
 
@@ -825,10 +907,10 @@ void mediamarks_init (void) {
 		      FALSE, FALSE, 0);
 
   cat_entry = gtk_entry_new ();
+  gtk_entry_set_activates_default (GTK_ENTRY(cat_entry), TRUE);
 /*
-  gtk_signal_connect (GTK_OBJECT(cat_entry), "activate",
-		      GTK_SIGNAL_FUNC (cat_close_cb),
-		      cat_dlg);
+  g_signal_connect (G_OBJECT(cat_entry), "activate",
+		    G_CALLBACK(cat_close_cb), cat_dlg);
 */
   gtk_box_pack_start_defaults (GTK_BOX (hbox), cat_entry);
 
@@ -849,4 +931,3 @@ void mediamarks_init (void) {
     se_defuns (gse, NULL, defs);
   }
 }
-
