@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  *
- * $Id: player.c,v 1.29.2.1 2005/04/23 16:49:54 dsalt Exp $
+ * $Id: player.c,v 1.56 2005/11/11 21:06:01 dsalt Exp $
  *
  * player object 
  */
@@ -30,6 +30,8 @@
 #include <gtk/gtkradiomenuitem.h>
 
 #include "globals.h"
+#include "menu.h"
+#include "ui.h"
 #include "utils.h"
 #include "playlist.h"
 #include "gtkvideo.h"
@@ -37,6 +39,9 @@
 #include "engine.h"
 #include "player.h"
 
+#define GXINE_VO_ZOOM_MIN (XINE_VO_ZOOM_MIN < 0 ? 0 : XINE_VO_ZOOM_MIN)
+#define GXINE_MAX_SPU_CHANNEL (31)
+
 xine_stream_t *stream; /* global stream */
 xine_audio_port_t *audio_port;
 xine_video_port_t *video_port;
@@ -51,61 +56,62 @@ static int             goto_pos, goto_time;
 static pthread_t       play_thread;
 static char           *cur_mrl = NULL, *cur_title = NULL;
 
+gboolean
+player_update_metadata (void)
+{
+  gdk_threads_enter();
+  infobar_show_metadata (infobars);
+  playlist_check_set_title ();
+  /*gdk_flush ();*/
+  gdk_threads_leave();
+  return FALSE;
+}
+
 static void *play_exec (void *queue_gen) {
 
   pthread_mutex_lock (&engine_lock);
 
   gdk_threads_enter();
-  infobar_clear (bar);
-  infobar_line (bar, 0, _("opening..."));
-  infobar_line (bar, 1, cur_mrl);
+  gxineinfo_clear (infobars);
+  gxineinfo_update_line (infobars, 0, "%s", _("opening..."));
+  gxineinfo_update_line (infobars, 1, "%s", cur_mrl);
   gdk_threads_leave();
 
-  if (!xine_open (stream, cur_mrl)) {
-
+  if (!xine_open (stream, cur_mrl))
+  {
+    int error;
     gdk_threads_enter();
-    switch (xine_get_error (stream)) {
+    switch (error = xine_get_error (stream)) {
     case XINE_ERROR_NO_INPUT_PLUGIN:
-
-      display_error (_("xine engine error"),
-		     _("xine engine failed to start.\n\n"
-		     "No input plugin found.\n"
-		     "Maybe the file does not exist, has wrong permissions or\n"
-		     "URL syntax error."));
-
+      display_error (FROM_XINE, _("The xine engine failed to start."),
+		     _("No input plugin was found.\n"
+		     "Maybe the file does not exist or cannot be accessed, "
+		     "or there is an error in the URL."));
       break;
+
     case XINE_ERROR_NO_DEMUX_PLUGIN:
-
-      display_error (_("xine engine error"),
-		     _("xine engine failed to start.\n\n"
-		     "No demuxer found - stream format not recognised."));
-
+      display_error (FROM_XINE, _("The xine engine failed to start."),
+		     _("No demuxer found - stream format not recognised."));
       break;
 
     case XINE_ERROR_MALFORMED_MRL:
-      display_error (_("xine engine error"),
-		     "xine engine failed to start.\n\n"
-		     "Malformed MRL.");
-
+      display_error (FROM_XINE, _("The xine engine failed to start."),
+		     _("The MRL is malformed."));
       break;
 
     default:
-      printf (_("error: %d\n"), xine_get_error (stream));
-      display_error (_("xine engine error"),
-		     _("xine engine failed to start.\n\n"
-		     "Unknown error."));
+      display_error (FROM_XINE, _("The xine engine failed to start."),
+		     _("Unknown error (code %d)"), error);
     }
 
-    infobar_clear (bar);
-    infobar_line (bar, 0, "gxine %s", VERSION);
-
+    g_idle_add ((GSourceFunc)playlist_logo, NULL);
     gdk_threads_leave();
     goto done;
   }
 
   gdk_threads_enter();
   ui_set_status (playlist_showing_logo () ? UI_STOP : UI_PLAY);
-  infobar_show_metadata ();
+  infobar_show_metadata (infobars);
   gdk_threads_leave();
 
   xine_play (stream, goto_pos, goto_time);
@@ -115,11 +121,12 @@ static void *play_exec (void *queue_gen) {
   /* correct for possible stream changes */
   gdk_threads_enter();
   ui_set_status (playlist_showing_logo () ? UI_STOP : UI_PLAY);
-  infobar_show_metadata ();
+  // gtk_timeout_add (500, G_CALLBACK(player_update_metadata), NULL);
+  infobar_show_metadata (infobars);
   playlist_check_set_title ();
-  gdk_threads_leave();
 
 done:
+  gdk_threads_leave();
   pthread_mutex_unlock (&engine_lock);
   pthread_exit (NULL);
   return NULL;
@@ -129,18 +136,20 @@ void player_launch (const char *title, const char *mrl, int pos, int pos_time)
 {
   int err;
 
-  goto_pos  = pos;
-  goto_time = pos_time;
+  goto_pos  = pos < 0 ? 0 : pos;
+  goto_time = pos_time < 0 ? 0 : pos_time;
   free (cur_title);
   free (cur_mrl);
   cur_title = title && !playlist_showing_logo () ? strdup (title) : NULL;
   cur_mrl = strdup (mrl);
 
+  gtk_video_unblank_screen (GTK_VIDEO (gtv));
+
   /* start separate thread, so xine_open will not block ui */
   if ((err = pthread_create (&play_thread,
 			     NULL, play_exec, NULL)) != 0) {
-    printf (_("playlist: can't create new thread (%s)\n"),
-	    strerror(err));
+    display_error_modal (FROM_GXINE, _("Player failure"),
+			 _("Can't create new thread: %s\n"), strerror(err));
     abort();
   }
 
@@ -148,6 +157,144 @@ void player_launch (const char *title, const char *mrl, int pos, int pos_time)
 }
 
 
+static void set_mute (int mute)
+{
+  ui_set_status (mute ? UI_AUDIO_MUTE : UI_AUDIO_UNMUTE);
+  xine_set_param (stream, XINE_PARAM_AUDIO_MUTE, mute);
+}
+
+static void set_speed (int speed)
+{
+  if (speed == XINE_SPEED_NORMAL)
+    ui_set_status (UI_PLAY);
+  else if (speed > XINE_SPEED_NORMAL)
+    ui_set_status (UI_FAST_FORWARD);
+  else if (speed)
+    ui_set_status (UI_PLAY_SLOW);
+  else
+    ui_set_status (UI_PAUSE);
+
+  xine_set_param (stream, XINE_PARAM_SPEED, speed);
+}
+
+static void set_zoom (int zoom)
+{
+  if (zoom < GXINE_VO_ZOOM_MIN)
+    zoom = GXINE_VO_ZOOM_MIN;
+  else if (zoom > XINE_VO_ZOOM_MAX)
+    zoom = XINE_VO_ZOOM_MAX;
+  xine_set_param (stream, XINE_PARAM_VO_ZOOM_X, zoom);
+  xine_set_param (stream, XINE_PARAM_VO_ZOOM_Y, zoom);
+}
+
+#define ACTION_SET(Func) \
+  static void set_##Func (int v) \
+  { \
+    if (gtk_toggle_action_get_active (action_items.Func) != v) \
+      gtk_action_activate (GTK_ACTION(action_items.Func)); \
+  }
+
+ACTION_SET(fullscreen)
+static void get_fullscreen (se_prop_t *prop, se_prop_read_t *value)
+{
+  value->i = gtk_video_is_fullscreen (GTK_VIDEO(gtv));
+}
+
+ACTION_SET(deinterlace)
+static void get_deinterlace (se_prop_t *prop, se_prop_read_t *value)
+{
+  value->i = gtk_video_get_use_post_plugins_deinterlace (GTK_VIDEO(gtv));
+}
+
+static void set_subtitles (int sub)
+{
+  if (sub >= -2 && sub <= GXINE_MAX_SPU_CHANNEL)
+  {
+    GSList *menuitem = action_items.subtitles;
+    int count = g_slist_length (menuitem) - 3;
+    if (count >= sub)
+    {    
+      while (menuitem && --count > sub)
+	menuitem = menuitem->next;
+      if (menuitem)
+      {
+        gtk_action_activate (menuitem->data);
+        return;
+      }
+    }
+    /* not in menu */
+    gtk_action_activate (g_slist_last (action_items.subtitles)->data);
+    xine_set_param (stream, XINE_PARAM_SPU_CHANNEL, sub);
+  }
+}
+
+static void set_aspect (int aspect)
+{
+  GSList *menuitem = action_items.aspect;
+  int count = g_slist_length (menuitem);
+  if (aspect >= XINE_VO_ASPECT_NUM_RATIOS)
+    aspect = XINE_VO_ASPECT_AUTO;
+  else if (aspect < 0)
+    aspect = XINE_VO_ASPECT_NUM_RATIOS - 1;
+  while (--count > aspect && menuitem->next)
+    menuitem = menuitem->next;
+  if (menuitem)
+    gtk_action_activate (menuitem->data);
+}
+
+ACTION_SET(auto_resize)
+static void get_auto_resize (se_prop_t *prop, se_prop_read_t *value)
+{
+  value->i = gtk_video_get_auto_resize (GTK_VIDEO(gtv));
+}
+
+ACTION_SET(auto_rescale)
+static void get_auto_rescale (se_prop_t *prop, se_prop_read_t *value)
+{
+  value->i = gtk_video_get_auto_rescale (GTK_VIDEO(gtv));
+}
+
+ACTION_SET(vo_postproc)
+static void get_vo_postproc (se_prop_t *prop, se_prop_read_t *value)
+{
+  value->i = gtk_video_get_use_post_plugins_video (GTK_VIDEO(gtv));
+}
+
+ACTION_SET(ao_postproc)
+static void get_ao_postproc (se_prop_t *prop, se_prop_read_t *value)
+{
+  value->i = gtk_video_get_use_post_plugins_audio (GTK_VIDEO(gtv));
+}
+
+static JSBool set_vo_postproc_chain (JSContext *cx, JSObject *obj,
+				     uintN argc, jsval *argv, jsval *rval)
+{
+  se_t *se = (se_t *) JS_GetContextPrivate(cx);
+
+  se_log_fncall ("vo_postproc.set_chain");
+  se_argc_check (1, "vo_postproc.set_chain");
+  se_arg_is_string (0, "vo_postproc.set_chain");
+
+  gtk_video_set_post_plugins_video
+    (GTK_VIDEO(gtv), JS_GetStringBytes (JS_ValueToString (cx, argv[0])));
+  return JS_TRUE;
+}
+
+static JSBool set_ao_postproc_chain (JSContext *cx, JSObject *obj,
+				     uintN argc, jsval *argv, jsval *rval)
+{
+  se_t *se = (se_t *) JS_GetContextPrivate(cx);
+
+  se_log_fncall ("ao_postproc.set_chain");
+  se_argc_check (1, "ao_postproc.set_chain");
+  se_arg_is_string (0, "ao_postproc.set_chain");
+
+  gtk_video_set_post_plugins_audio
+    (GTK_VIDEO(gtv), JS_GetStringBytes (JS_ValueToString (cx, argv[0])),
+     audio_port);
+  return JS_TRUE;
+}
+
 static JSBool get_time (JSContext *cx, JSObject *obj, uintN argc, 
 		     jsval *argv, jsval *rval) {
 
@@ -163,6 +310,8 @@ static JSBool get_time (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
+#ifdef WITH_DEPRECATED
+
 static JSBool js_get_speed (JSContext *cx, JSObject *obj, uintN argc, 
 			    jsval *argv, jsval *rval) {
 
@@ -219,17 +368,19 @@ static JSBool js_get_zoom (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
+#endif /* WITH_DEPRECATED */
+
 static JSBool controls_pause (JSContext *cx, JSObject *obj, uintN argc, 
 			      jsval *argv, jsval *rval) {
 
   se_t *se = (se_t *) JS_GetContextPrivate(cx);
   int   pause_state=-1;
 
-  se_log_fncall ("pause");
+  se_log_fncall_checkinit ("pause");
   se_argc_check_max (1, "pause");
 
   if (argc==1) {
-    se_arg_is_int (0, "pause");
+    se_arg_is_int_or_bool (0, "pause");
     JS_ValueToInt32 (cx, argv[0], &pause_state);
   }
 
@@ -268,7 +419,7 @@ static JSBool controls_stop (JSContext *cx, JSObject *obj, uintN argc,
 
   se_t *se = (se_t *) JS_GetContextPrivate(cx);
 
-  se_log_fncall ("stop");
+  se_log_fncall_checkinit ("stop");
 
   if (engine_mutex_trylock (se))
     return JS_TRUE;
@@ -282,6 +433,8 @@ static JSBool controls_stop (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
+#ifdef WITH_DEPRECATED
+
 static JSBool js_set_speed (JSContext *cx, JSObject *obj, uintN argc, 
 			    jsval *argv, jsval *rval) {
 
@@ -300,20 +453,8 @@ static JSBool js_set_speed (JSContext *cx, JSObject *obj, uintN argc,
 
   if (engine_mutex_trylock (se))
     return JS_TRUE;
-
-  if (speed == XINE_SPEED_NORMAL)
-    ui_set_status (UI_PLAY);
-  else if (speed > XINE_SPEED_NORMAL)
-    ui_set_status (UI_FAST_FORWARD);
-  else if (speed)
-    ui_set_status (UI_PLAY_SLOW);
-  else
-    ui_set_status (UI_PAUSE);
-
-  xine_set_param (stream, XINE_PARAM_SPEED, speed);
-
+  set_speed (speed);
   pthread_mutex_unlock (&engine_lock);
-
   return JS_TRUE;
 }
 
@@ -354,7 +495,7 @@ static JSBool js_set_mute (JSContext *cx, JSObject *obj, uintN argc,
 
   if (argc == 1)
   {
-    se_arg_is_int (0, "set_mute");
+    se_arg_is_int_or_bool (0, "set_mute");
     JS_ValueToBoolean (cx, argv[0], &mute);
   }
   else
@@ -362,10 +503,8 @@ static JSBool js_set_mute (JSContext *cx, JSObject *obj, uintN argc,
 
   if (engine_mutex_trylock (se))
     return JS_TRUE;
-  ui_set_status (mute ? UI_AUDIO_MUTE : UI_AUDIO_UNMUTE);
-  xine_set_param (stream, XINE_PARAM_AUDIO_MUTE, mute);
+  set_mute (mute);
   pthread_mutex_unlock (&engine_lock);
-
   return JS_TRUE;
 }
 
@@ -381,29 +520,24 @@ static JSBool js_set_zoom (JSContext *cx, JSObject *obj, uintN argc,
 
   JS_ValueToInt32 (cx, argv[0], &zoom);
 
-  if (zoom<XINE_VO_ZOOM_MIN)
-    zoom = XINE_VO_ZOOM_MIN;
-  if (zoom>XINE_VO_ZOOM_MAX)
-    zoom = XINE_VO_ZOOM_MAX;
-
   se_log ("zoom is %d\n", zoom);
 
   if (engine_mutex_trylock (se))
     return JS_TRUE;
-
-  xine_set_param (stream, XINE_PARAM_VO_ZOOM_X, zoom);
-  xine_set_param (stream, XINE_PARAM_VO_ZOOM_Y, zoom);
-
+  set_zoom (zoom);
   pthread_mutex_unlock (&engine_lock);
 
   return JS_TRUE;
 }
 
+#endif /* WITH_DEPRECATED */
+
 static JSBool js_set_video_size (JSContext *cx, JSObject *obj, uintN argc, 
 			   jsval *argv, jsval *rval) {
 
   se_t *se = (se_t *) JS_GetContextPrivate(cx);
   int   zoom;
+  GtkAction *action;
 
   se_log_fncall ("set_video_size");
   se_argc_check (1, "set_video_size");
@@ -411,31 +545,26 @@ static JSBool js_set_video_size (JSContext *cx, JSObject *obj, uintN argc,
 
   JS_ValueToInt32 (cx, argv[0], &zoom);
 
-  if (zoom<XINE_VO_ZOOM_MIN)
-    zoom = XINE_VO_ZOOM_MIN;
-  if (zoom>XINE_VO_ZOOM_MAX)
+  if (zoom<GXINE_VO_ZOOM_MIN)
+    zoom = GXINE_VO_ZOOM_MIN;
+  else if (zoom>XINE_VO_ZOOM_MAX)
     zoom = XINE_VO_ZOOM_MAX;
-  gtk_video_rescale (GTK_VIDEO(gtv), (double) zoom / 100.0);
+
+  /* find & activate an action item which corresponds to the requested scale */
+  action = find_video_size_action (zoom);
+  if (action)
+    gtk_action_activate (action);
+  else
+  {
+    scale_changed_cb (NULL, zoom, NULL);
+    do_pending_events ();
+    gtk_video_rescale ((GtkVideo *) gtv, zoom);
+  }
 
   return JS_TRUE;
 }
 
-static JSBool js_set_sub (JSContext *cx, JSObject *obj, uintN argc, 
-			  jsval *argv, jsval *rval) {
-
-  se_t *se = (se_t *) JS_GetContextPrivate(cx);
-  int   channel;
-
-  se_log_fncall ("set_sub");
-  se_argc_check (1, "set_sub");
-  se_arg_is_int (0, "set_sub");
-
-  JS_ValueToInt32 (cx, argv[0], &channel);
-
-  xine_set_param (stream, XINE_PARAM_SPU_CHANNEL, channel);
-
-  return JS_TRUE;
-}
+#ifdef WITH_DEPRECATED
 
 static JSBool js_set_fullscreen (JSContext *cx, JSObject *obj, uintN argc, 
 				 jsval *argv, jsval *rval) {
@@ -447,20 +576,13 @@ static JSBool js_set_fullscreen (JSContext *cx, JSObject *obj, uintN argc,
   se_argc_check_max (1, "set_fullscreen");
 
   if (argc==1) {
-    se_arg_is_int (0, "set_fullscreen");
+    se_arg_is_int_or_bool (0, "set_fullscreen");
     JS_ValueToInt32 (cx, argv[0], &fs);
   } else {
     fs = !gtk_video_is_fullscreen (GTK_VIDEO(gtv));
   }
 
-  fullscreen_menu_item[0]->active = fs;
-  fullscreen_menu_item[1]->active = fs;
-  gtk_video_set_fullscreen (GTK_VIDEO(gtv), fs);
-  if (fs)
-    window_toolbar_restore ();
-  else
-    window_toolbar_reset ();
-
+  set_fullscreen (fs);
   return JS_TRUE;
 }
 
@@ -474,18 +596,13 @@ static JSBool js_set_deinterlace (JSContext *cx, JSObject *obj, uintN argc,
   se_argc_check_max (1, "set_deinterlace");
 
   if (argc==1) {
-    se_arg_is_int (0, "set_deinterlace");
+    se_arg_is_int_or_bool (0, "set_deinterlace");
     JS_ValueToInt32 (cx, argv[0], &di);
   } else {
-    di = !gtk_video_get_deinterlace (GTK_VIDEO(gtv));
+    di = !gtk_video_get_use_post_plugins_deinterlace (GTK_VIDEO(gtv));
   }
 
-  gtk_video_set_deinterlace (GTK_VIDEO(gtv), di);
-
-  /* Hmm, should be a don't-emit-signal method for this... */
-  deinterlace_menu_item[0]->active = di;
-  deinterlace_menu_item[1]->active = di;
-
+  set_deinterlace (di);
   return JS_TRUE;
 }
 
@@ -518,12 +635,7 @@ static JSBool js_set_postproc_video (JSContext *cx, JSObject *obj,
   } else
     di = !gtk_video_get_use_post_plugins_video (GTK_VIDEO(gtv));
 
-  gtk_video_set_use_post_plugins_video (GTK_VIDEO(gtv), di);
-
-  /* Hmm, should be a don't-emit-signal method for this... */
-  pp_video_menu_item[0]->active = di;
-  pp_video_menu_item[1]->active = di;
-
+  set_vo_postproc (di);
   return JS_TRUE;
 }
 
@@ -557,12 +669,7 @@ static JSBool js_set_postproc_audio (JSContext *cx, JSObject *obj,
   } else
     di = !gtk_video_get_use_post_plugins_audio (GTK_VIDEO(gtv));
 
-  gtk_video_set_use_post_plugins_audio (GTK_VIDEO(gtv), di, audio_port);
-
-  /* Hmm, should be a don't-emit-signal method for this... */
-  pp_audio_menu_item[0]->active = di;
-  pp_audio_menu_item[1]->active = di;
-
+  set_ao_postproc (di);
   return JS_TRUE;
 }
 
@@ -576,16 +683,13 @@ static JSBool js_set_auto_resize (JSContext *cx, JSObject *obj, uintN argc,
   se_argc_check_max (1, "set_auto_resize");
 
   if (argc==1) {
-    se_arg_is_int (0, "set_auto_resize");
+    se_arg_is_int_or_bool (0, "set_auto_resize");
     JS_ValueToInt32 (cx, argv[0], &ar);
   } else {
     ar = !gtk_video_get_auto_resize (GTK_VIDEO(gtv));
   }
 
-  gtk_video_set_auto_resize (GTK_VIDEO(gtv), ar);
-  auto_resize_menu_item[0]->active = ar;
-  auto_resize_menu_item[1]->active = ar;
-
+  set_auto_resize (ar);
   return JS_TRUE;
 }
 
@@ -599,24 +703,21 @@ static JSBool js_set_auto_rescale (JSContext *cx, JSObject *obj, uintN argc,
   se_argc_check_max (1, "set_auto_rescale");
 
   if (argc==1) {
-    se_arg_is_int (0, "set_auto_rescale");
+    se_arg_is_int_or_bool (0, "set_auto_rescale");
     JS_ValueToInt32 (cx, argv[0], &ar);
   } else {
     ar = !gtk_video_get_auto_rescale (GTK_VIDEO(gtv));
   }
 
-  gtk_video_set_auto_rescale (GTK_VIDEO(gtv), ar);
-  auto_rescale_menu_item[0]->active = ar;
-  auto_rescale_menu_item[1]->active = ar;
-
+  set_auto_rescale (ar);
   return JS_TRUE;
 }
 
 static JSBool js_set_aspect (JSContext *cx, JSObject *obj, uintN argc, 
-			     jsval *argv, jsval *rval) {
-
+			     jsval *argv, jsval *rval)
+{
   se_t *se = (se_t *) JS_GetContextPrivate(cx);
-  int i, aspect, count = g_slist_length (aspect_menu_items[0]);
+  int aspect;
 
   se_log_fncall ("set_aspect");
   se_argc_check_max (1, "set_aspect");
@@ -624,49 +725,34 @@ static JSBool js_set_aspect (JSContext *cx, JSObject *obj, uintN argc,
   if (argc==1) {
     se_arg_is_int (0, "set_aspect");
     JS_ValueToInt32 (cx, argv[0], &aspect);
-  } else {
-    aspect = xine_get_param (stream, XINE_PARAM_VO_ASPECT_RATIO);
-    if (aspect >= count)
-      aspect = XINE_VO_ASPECT_AUTO;
-    else
-      aspect = (aspect + 1) % count;
-  }
-
-  xine_set_param (stream, XINE_PARAM_VO_ASPECT_RATIO, aspect);
-
-  for (i = 0; i < 2; ++i)
-  {
-    GSList *menuitem = aspect_menu_items[i];
-    while (--count >= 0)
-    {
-      /* Hmm, should be a don't-emit-signal method for this... */
-      GTK_CHECK_MENU_ITEM (menuitem->data)->active = (aspect == count);
-      menuitem = menuitem->next;
-    }
-  }
+  } else
+    aspect = xine_get_param (stream, XINE_PARAM_VO_ASPECT_RATIO) + 1;
 
+  set_aspect (aspect);
   return JS_TRUE;
 }
 
+#endif /* WITH_DEPRECATED */
+
 static JSBool js_snapshot (JSContext *cx, JSObject *obj, uintN argc, 
 			   jsval *argv, jsval *rval) {
 
   se_t     *se = (se_t *) JS_GetContextPrivate(cx);
   JSString *str;
   char     *fname = NULL;
-  int	    scale = -1, blend = -1;
+  int32	    scale = -1, blend = -1;
 
-  se_log_fncall ("snapshot");
+  se_log_fncall_checkinit ("snapshot");
   se_argc_check_range (0, 3, "snapshot");
 
   switch (argc) /* note fall-through */
   {
   case 3:
     se_arg_is_int (2, "snapshot");
-    JS_ValueToInt32 (cx, argv[0], &blend);
+    JS_ValueToInt32 (cx, argv[2], &blend);
   case 2:
     se_arg_is_int (1, "snapshot");
-    JS_ValueToInt32 (cx, argv[0], &scale);
+    JS_ValueToInt32 (cx, argv[1], &scale);
   case 1:
     se_arg_is_string (0, "snapshot");
     str = JS_ValueToString (cx, argv[0]);
@@ -758,7 +844,7 @@ static JSBool controls_play (JSContext *cx, JSObject *obj, uintN argc,
   char *mrl = NULL;
   int  pos, pos_time;
 
-  se_log_fncall ("play");
+  se_log_fncall_checkinit ("play");
   se_argc_check_max (3, "play");
 
   pos = -1; pos_time = -1;
@@ -807,54 +893,38 @@ static JSBool controls_play (JSContext *cx, JSObject *obj, uintN argc,
     break;
   }
 
-  pos *=655 ;
+  if (pos > 0)
+    pos *= 655;
 
   if (engine_mutex_trylock (se))
     return JS_TRUE;
 
-  if (mrl) {
+  if (mrl)
+    playlist_play_from (playlist_add_mrl (mrl, -1), pos, pos_time);
+  else
+  {
+    play_item_t *item = playlist_get_current_item ();
 
-    int item;
-
-    item = playlist_add_mrl (mrl, -1);
-
-    if (pos<0)
-      pos = 0;
-    if (pos_time<0)
-      pos_time = 0;
-
-    playlist_play_from (item, pos, pos_time);
-
-  } else {
-
-    play_item_t *item;
-
-    item = playlist_get_item (playlist_get_list_pos());
-
-    if (item) {
+    if (item)
+    {
       switch (xine_get_status (stream))
       {
       case XINE_STATUS_STOP:
       case XINE_STATUS_IDLE:
-	if (pos<0)
-	  pos = 0;
-	if (pos_time<0)
-	  pos_time = item->start_time;
-	playlist_play_from (playlist_get_list_pos(), pos, pos_time);
+	playlist_play_from (playlist_get_list_pos (), pos,
+			   pos_time < 0 ? item->start_time : pos_time);
 	break;
       default:
-	if ( (pos>=0) || (pos_time>=0) )
+	if (pos >= 0 || pos_time >= 0)
 	{
-	  if (pos<0)
-	    pos = 0;
-	  if (pos_time<0)
-	    pos_time = 0;
-	  xine_play (stream, pos, pos_time);
-	}
+	  gtk_video_unblank_screen (GTK_VIDEO (gtv));
+	  xine_play (stream, pos < 0 ? 0 : pos, pos_time < 0 ? 0 : pos_time);
+        }
 	break;
       }
+      play_item_dispose (item);
     } else
-      printf (_("script_engine: error, no valid play item available\n"));
+      logprintf (_("script_engine: error, no valid play item available\n"));
   } 
 
   if (xine_get_param (stream, XINE_PARAM_SPEED) != XINE_SPEED_NORMAL) {
@@ -875,11 +945,11 @@ static JSBool controls_play_pause (JSContext *cx, JSObject *obj, uintN argc,
 
   se_t *se = (se_t *) JS_GetContextPrivate(cx);
 
-  se_log_fncall ("play_pause");
+  se_log_fncall_checkinit ("play_pause");
   se_argc_check_max (1, "play_pause");
 
   if (argc==1) {
-    se_arg_is_int (0, "play_pause");
+    se_arg_is_int_or_bool (0, "play_pause");
   }
 
   switch (xine_get_status (stream))
@@ -964,17 +1034,20 @@ static xine_audio_port_t *load_audio_out_driver (void) {
     if (audio_port)
       return audio_port;
     else
-      printf (_("audio driver %s failed\n"), audio_driver_id);
+      fprintf (stderr, _("audio driver %s failed\n"), audio_driver_id);
   }
  
   /* autoprobe */
   return xine_open_audio_driver (xine, NULL, NULL);
 }
 
+/* Properties */
 
-void player_init (void) {
-
+void player_init (void)
+{
+/*
   se_o_t            *controls_obj;
+ */
   
   cur_mrl = NULL;
 
@@ -989,10 +1062,53 @@ void player_init (void) {
    * set up script engine objects and functions 
    */
 
+/*
   controls_obj = se_create_object (gse, NULL, "controls", NULL);
 
   se_defun (gse, controls_obj, "play", controls_play, 0, 0,
 	    SE_GROUP_HIDDEN, NULL, NULL);
+ */
+
+  {
+    static const ui_property_t int_props[] = {
+      /* Text is help for the commands. Format is:
+       *   To the left of the semicolon, a comma-separated list of
+       *     VARIABLE[=TYPE] or METHOD([PARAMETER TYPE,...])
+       *   To the right, descriptive text
+       */
+      { "av_speed", N_("v=int, min, max; playback speed"), XINE_PARAM_SPEED, NULL, NULL, XINE_SPEED_PAUSE, XINE_SPEED_FAST_4 },
+      { "vo_aspect", N_("v=int; aspect ratio"), XINE_PARAM_VO_ASPECT_RATIO, NULL, set_aspect, 0, XINE_VO_ASPECT_NUM_RATIOS - 1, TRUE },
+      { "vo_subtitle", N_("v=int, min, max; subtitle channel"), XINE_PARAM_SPU_CHANNEL, NULL, set_subtitles, -2, GXINE_MAX_SPU_CHANNEL },
+      { "vo_zoom", N_("v=int, min, max; video zoom (within window)"), XINE_PARAM_VO_ZOOM_X, NULL, set_zoom, XINE_VO_ZOOM_MIN, XINE_VO_ZOOM_MAX },
+      { NULL }
+    };
+    static const ui_property_t bool_props[] = {
+      { "ao_mute", N_("v=bool, toggle(): audio mute"), XINE_PARAM_AUDIO_MUTE, NULL, set_mute },
+      { "vo_auto_resize", N_("v=bool, toggle(): auto-resize video on frame shape change"), 0, get_auto_resize, set_auto_resize },
+      { "vo_auto_rescale", N_("v=bool, toggle(): double size of low-resolution video"), 0, get_auto_rescale, set_auto_rescale },
+      { "vo_fullscreen", N_("v=bool, toggle(): full-screen mode"), 0, get_fullscreen, set_fullscreen },
+      { "vo_deinterlace", N_("v=bool, toggle(): enable deinterlacing"), 0, get_deinterlace, set_deinterlace },
+      { "ao_postproc", N_("v=bool, toggle(): audio post-processing; set_chain(string): plugin chain"), 0, get_ao_postproc, set_ao_postproc },
+      { "vo_postproc", N_("v=bool, toggle(): video post-processing; set_chain(string): plugin chain"), 0, get_vo_postproc, set_vo_postproc },
+      { NULL }
+    };
+    static const ui_property_t zoom_subprops[] = {
+      { "x", N_("v=int, min, max; video zoom (within window)"), XINE_PARAM_VO_ZOOM_X, NULL, NULL, XINE_VO_ZOOM_MIN, XINE_VO_ZOOM_MAX },
+      { "y", N_("v=int, min, max; video zoom (within window)"), XINE_PARAM_VO_ZOOM_Y, NULL, NULL, XINE_VO_ZOOM_MIN, XINE_VO_ZOOM_MAX },
+      { NULL }
+    };
+
+    ui_create_properties (int_props, NULL, SE_TYPE_INT, NULL);
+    ui_create_properties (bool_props, NULL, SE_TYPE_BOOL, NULL);
+    ui_create_properties (zoom_subprops, se_find_object (gse, NULL, "vo_zoom"),
+			  SE_TYPE_INT, NULL);
+
+    /* a few extras - chain definition should be done via the GUI */
+    se_defun (gse, se_find_object (gse, NULL, "ao_postproc"), "set_chain",
+	      set_ao_postproc_chain, 0, 0, SE_GROUP_HIDDEN, NULL, NULL);
+    se_defun (gse, se_find_object (gse, NULL, "vo_postproc"), "set_chain",
+	      set_vo_postproc_chain, 0, 0, SE_GROUP_HIDDEN, NULL, NULL);
+  }
 
   /*
    * hook up global xine functions for convenience
@@ -1001,6 +1117,7 @@ void player_init (void) {
   {
     static const se_f_def_t defs[] = {
       { "play", controls_play, 0, 0,
+	/* help text describes function parameters */
 	SE_GROUP_ENGINE, N_("[mrl] [, pos, time]"),
 	N_("time in milliseconds") },
       { "get_time", get_time, 0, 0,
@@ -1012,42 +1129,43 @@ void player_init (void) {
       { "play_pause", controls_play_pause, 0, 0,
 	SE_GROUP_ENGINE, N_("[bool]"), NULL },
       { "snapshot", js_snapshot, 0, 0,
-	SE_GROUP_ENGINE, N_("[file name]"), NULL },
+	SE_GROUP_ENGINE, N_("[file name [, scale, blend]]"), NULL },
 
-      { "set_speed", js_set_speed, 0, 0,
-	SE_GROUP_PROPERTIES, N_("int"), NULL },
-      { "get_speed", js_get_speed, 0, 0,
-	SE_GROUP_PROPERTIES, NULL, NULL },
-      { "set_volume", js_set_volume, 0, 0,
-	SE_GROUP_PROPERTIES, N_("int"), NULL },
-      { "get_volume", js_get_volume, 0, 0,
-	SE_GROUP_PROPERTIES, NULL, NULL },
-      { "set_mute", js_set_mute, 0, 0,
-	SE_GROUP_PROPERTIES, N_("[bool]"), NULL },
-      { "get_mute", js_get_mute, 0, 0,
-	SE_GROUP_PROPERTIES, NULL, NULL },
-      { "set_zoom", js_set_zoom, 0, 0,
-	SE_GROUP_PROPERTIES, N_("int"), NULL },
-      { "get_zoom", js_get_zoom, 0, 0,
-	SE_GROUP_PROPERTIES, NULL, NULL },
       { "set_video_size", js_set_video_size, 0, 0,
 	SE_GROUP_PROPERTIES, N_("int"), NULL },
+
+#ifdef WITH_DEPRECATED
+      { "set_speed", js_set_speed, 0, 0,
+	SE_GROUP_HIDDEN, NULL, NULL },
+      { "get_speed", js_get_speed, 0, 0,
+	SE_GROUP_HIDDEN, NULL, NULL },
+      { "set_volume", js_set_volume, 0, 0,
+	SE_GROUP_HIDDEN, NULL, NULL },
+      { "get_volume", js_get_volume, 0, 0,
+	SE_GROUP_HIDDEN, NULL, NULL },
+      { "set_mute", js_set_mute, 0, 0,
+	SE_GROUP_HIDDEN, NULL, NULL },
+      { "get_mute", js_get_mute, 0, 0,
+	SE_GROUP_HIDDEN, NULL, NULL },
+      { "set_zoom", js_set_zoom, 0, 0,
+	SE_GROUP_HIDDEN, NULL, NULL },
+      { "get_zoom", js_get_zoom, 0, 0,
+	SE_GROUP_HIDDEN, NULL, NULL },
       { "set_fullscreen", js_set_fullscreen, 0, 0,
-	SE_GROUP_PROPERTIES, N_("[bool]"), NULL },
+	SE_GROUP_HIDDEN, NULL, NULL },
       { "set_aspect", js_set_aspect, 0, 0,
-	SE_GROUP_PROPERTIES, N_("[bool]"), NULL },
-      { "set_sub", js_set_sub, 0, 0,
-	SE_GROUP_PROPERTIES, N_("int"), NULL },
+	SE_GROUP_HIDDEN, NULL, NULL },
       { "set_deinterlace", js_set_deinterlace, 0, 0,
-	SE_GROUP_PROPERTIES, N_("[bool]"), NULL },
+	SE_GROUP_HIDDEN, NULL, NULL },
       { "set_postproc_video", js_set_postproc_video, 0, 0,
-	SE_GROUP_PROPERTIES, N_("[bool|string]"), NULL },
+	SE_GROUP_HIDDEN, NULL, NULL },
       { "set_postproc_audio", js_set_postproc_audio, 0, 0,
-	SE_GROUP_PROPERTIES, N_("[bool|string]"), NULL },
+	SE_GROUP_HIDDEN, NULL, NULL },
       { "set_auto_resize", js_set_auto_resize, 0, 0,
-	SE_GROUP_PROPERTIES, N_("[bool]"), NULL },
+	SE_GROUP_HIDDEN, NULL, NULL },
       { "set_auto_rescale", js_set_auto_rescale, 0, 0,
-	SE_GROUP_PROPERTIES, N_("[bool]"), N_("magnify low-res video") },
+	SE_GROUP_HIDDEN, NULL, NULL },
+#endif /* WITH_DEPRECATED */
 
       { "is_live_stream", js_is_live_stream, 0, 0,
 	SE_GROUP_PROPERTIES, NULL, NULL },
