@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: vis.h,v 1.3 2005/01/13 19:52:23 dsalt Exp $
+ * $Id: vis.h,v 1.6 2005/05/11 20:13:19 dsalt Exp $
  *
  * Visualisation wrapper code
  */
@@ -23,9 +23,14 @@
 #ifndef HAVE_VIS_H
 #define HAVE_VIS_H
 
+#include <gtk/gtkradioaction.h>
+
+#include "gtkvideo.h"
+
 #define vis_hide(V,A) gtk_video_set_vis ((V), (A), FALSE)
 gboolean vis_show (GtkVideo *, xine_audio_port_t **);
-#define vis_set(V,ID,A) gtk_video_select_vis ((V), (ID), (A))
+void vis_set (const char *);
+void vis_cb (GtkRadioAction *, gpointer);
 
 void vis_init (void);
 
